#include "BaseSerialPort.h"

#include <iostream>
#include <sstream>
#include <string>

using namespace std;

#define MOD_LOG_TAG "libgtsdev.so"
#define _WANIN_LOG_
#ifdef  _WANIN_LOG_
    #include <android/log.h>
    #define LOG(fmt, args...)   __android_log_print(ANDROID_LOG_DEBUG, MOD_LOG_TAG, fmt, ##args)
    #define LOGI(fmt, args...)  __android_log_print(ANDROID_LOG_INFO, MOD_LOG_TAG, fmt, ##args)
    #define LOGE(fmt, args...)  __android_log_print(ANDROID_LOG_ERROR,MOD_LOG_TAG, fmt, ##args)
#else
    #define LOG (...)  
    #define LOGI(...)  
    #define LOGE(...)  
#endif

BaseSerialPort::BaseSerialPort()
:m_bRunning(false)
,m_type(SP_Default)
,m_iFd(-1)
,m_thread_recver(0)
{
	
}

BaseSerialPort::~BaseSerialPort()
{
	
}

bool BaseSerialPort::Start(int iPort, int iBaudRate, SerialPortType type)
{
	m_type = type;
	//LOGE("BaseSerialPort::Start Port%d, type: %d", iPort, type);
	
	if(!OpenPort(iPort))
	{
		return false;
	}
	if(!SetParams(iBaudRate, 8, 'N', 1))
	{
		close(m_iFd);
		return false;
	}
	
	m_bRunning = true;
	
	StartRecver();
	
	return true;
}

void BaseSerialPort::Stop()
{
    m_bRunning = false;
	if(m_thread_recver != 0)
	{
		void *dummy;
		pthread_join(m_thread_recver, &dummy);
		m_thread_recver = 0;
	}
	
	if(m_iFd > 0)
	{
		close(m_iFd);
		m_iFd = -1;
	}
}

bool BaseSerialPort::OpenPort(int port)
{
	std::string strDev = "/dev/ttyS";
	std::string strPort;
	std::stringstream ss;
	ss << port;
	ss >> strPort;
	strDev += strPort;
	long vdisable;
	m_iFd = open(strDev.c_str(), O_RDWR | O_NOCTTY | O_NDELAY);
	if (-1 == m_iFd)
	{
		LOGE("Can't Open Serial Port%d", port);
		return false;
	}
	
    if(fcntl(m_iFd, F_SETFL, 0) < 0)
    {
        LOGE("fcntl failed!");
    }
    //else
    //{
    //    LOGE("fcntl=%d",fcntl(fd, F_SETFL,0));
    //}
    if(isatty(STDIN_FILENO)==0)
    {
        LOGE("standard input is not a terminal device");
    }
    return true;
}

bool BaseSerialPort::SetParams(int nSpeed, int nBits, char nEvent, int nStop)
{
	struct termios newtio,oldtio;
	if(tcgetattr(m_iFd, &oldtio)  !=  0) 
	{ 
		LOGE("SetupSerial failed");
		return false;
	}
	bzero( &newtio, sizeof( newtio ) );
	newtio.c_cflag |= CLOCAL | CREAD; 
	newtio.c_cflag &= ~CSIZE; 

	switch( nBits )
	{
	case 7:
		newtio.c_cflag |= CS7;
		break;
	case 8:
		newtio.c_cflag |= CS8;
		break;
	}

	switch( nEvent )
	{
	case 'O':                     //奇校验
		newtio.c_cflag |= PARENB;
		newtio.c_cflag |= PARODD;
		newtio.c_iflag |= (INPCK | ISTRIP);
		break;
	case 'E':                     //偶校验
		newtio.c_iflag |= (INPCK | ISTRIP);
		newtio.c_cflag |= PARENB;
		newtio.c_cflag &= ~PARODD;
		break;
	case 'N':                    //无校验
		newtio.c_cflag &= ~PARENB;
		break;
	}

	switch( nSpeed )
	{
	case 2400:
		cfsetispeed(&newtio, B2400);
		cfsetospeed(&newtio, B2400);
		break;
	case 4800:
		cfsetispeed(&newtio, B4800);
		cfsetospeed(&newtio, B4800);
		break;
	case 9600:
		cfsetispeed(&newtio, B9600);
		cfsetospeed(&newtio, B9600);
		break;
	case 115200:
		cfsetispeed(&newtio, B115200);
		cfsetospeed(&newtio, B115200);
		break;
	default:
		cfsetispeed(&newtio, B9600);
		cfsetospeed(&newtio, B9600);
		break;
	}
	if( nStop == 1 )
	{
		newtio.c_cflag &=  ~CSTOPB;
	}
	else if ( nStop == 2 )
	{
		newtio.c_cflag |=  CSTOPB;
	}
	newtio.c_cc[VTIME] = 0;
	newtio.c_cc[VMIN] = 0;
	tcflush(m_iFd, TCIFLUSH);
	if((tcsetattr(m_iFd, TCSANOW, &newtio)) != 0)
	{
		LOGE("com set error");
		return false;
	}
	return true;
}

void BaseSerialPort::StartRecver()
{
	pthread_attr_t attr;
	pthread_attr_init(&attr);
	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
	pthread_create(&m_thread_recver, &attr, RecverWrapper, this);
	pthread_attr_destroy(&attr);
}

void * BaseSerialPort::RecverWrapper(void * p)
{
	BaseSerialPort * pThis = static_cast<BaseSerialPort *>(p);
	if(pThis->m_type == FarReader)
	{
		return (void *)pThis->FarRecverProcess();
	}
	else if(pThis->m_type == NearReader)
	{
		return (void *)pThis->NearRecverProcess();
	}
	else if(pThis->m_type == RelayVoltage)
	{
		return (void *)pThis->RelayVoltageRecverProcess();
	}
	return 0;
}

/*
数据格式： (近程)
STX(0X02) DATA(8HEX) CR LF ETX(0X03)
注：
STX 	为开始字符 （十六进制）
DATA	为8位十六进制字符卡号
CR 		为0D(表示空格)
LF 		为0A（表示换行）
ETX 	为结束字符03（十六进制）
实例：读卡器输出到计算机十六进制卡号类型：
  02 30 31 38 31 30 31 44 34 0D 0A 03 
其中：30 31 38 31 30 31 44 34 为卡号的四个字节的ASCII码
*/
int BaseSerialPort::NearRecverProcess()
{
	LOGE("enter NearRecverProcess");
	char szBuf[256] = {0};
	fd_set readset; 
	struct timeval tv;
	int ret = -1;
	int size = 0;
	char szTmp[4] = {0};
	int iBase = 0, iCardNum = 0;
	while(m_bRunning)
	{
		FD_ZERO(&readset);
		FD_SET(m_iFd, &readset);
		tv.tv_sec = 0;
		tv.tv_usec = 1000 * 1000;
		
		ret = select(m_iFd + 1, &readset, NULL, NULL, &tv);
		if(ret < 0)
		{
			LOGE("read port error");
			break;
		}
		
		if(ret == 0)
		{
			continue;
		}

		if(FD_ISSET(m_iFd, &readset))
		{
			size = read(m_iFd, szBuf, 14);
			//LOGE("szBuf[0]=%02x, szBuf[1]=%02x, szBuf[2]=%02x, szBuf[3]=%02x, szBuf[4]=%02x", szBuf[0], szBuf[1], szBuf[2], szBuf[3], szBuf[4]);
			//LOGE("szBuf[5]=%02x, szBuf[6]=%02x, szBuf[7]=%02x, szBuf[8]=%02x, szBuf[9]=%02x, szBuf[10]=%02x, iCardNum = %d", szBuf[5], szBuf[6], szBuf[7], szBuf[8], szBuf[9], szBuf[10], iCardNum);
			
			//size = read(m_iFd, szBuf, 14);
			if(size > 0)
			{
				memset(szTmp, 0, 4);
				/*
				szTmp[0] = szBuf[5];
				szTmp[1] = szBuf[6];
				sscanf(szTmp, "%02x", &iBase);
				szTmp[0] = szBuf[7];
				szTmp[1] = szBuf[8];
				szTmp[2] = szBuf[9];
				szTmp[3] = szBuf[10];
				sscanf(szTmp, "%04x", &iCardNum);
				LOGE("base = %d, num = %d", iBase, iCardNum);
				iCardNum = iBase * 100000 + iCardNum;
				*/
				szTmp[0] = szBuf[3];
				szTmp[1] = szBuf[4];
				sscanf(szTmp, "%02x", &iBase);
				szTmp[0] = szBuf[5];
				szTmp[1] = szBuf[6];
				szTmp[2] = szBuf[7];
				szTmp[3] = szBuf[8];
				sscanf(szTmp, "%04x", &iCardNum);
				//LOGE("base = %d, num = %d", iBase, iCardNum);
				iCardNum = iBase * 100000 + iCardNum;
				SendCardNumber(iCardNum, NearReader);
				
				//LOGE("szBuf[5]=%02x, szBuf[6]=%02x, szBuf[7]=%02x, szBuf[8]=%02x, szBuf[9]=%02x, szBuf[10]=%02x, iCardNum = %d", szBuf[5], szBuf[6], szBuf[7], szBuf[8], szBuf[9], szBuf[10], iCardNum);
			}
		}
	}
	
	return 1;
}

/*
数据格式： (远程)
起始码　0x30 0x32
读卡器id　1-255, ２字节发送，ASCII码 
数据　5个字节的卡号，分10字节ASCII码发送
校验crc 取5个字节卡号之和的最低字节，分2字节ASCII码发送
结束码　0x0D 0X0A
*/
int BaseSerialPort::FarRecverProcess()
{
	LOGE("enter FarRecverProcess");
	char szBuf[256] = {0};
	char szTmp[4] = {0};
	fd_set readset; 
	struct timeval tv;
	int ret = -1;
	int size = 0;
	int pos = 0;
	int idx = 0;
	int iState = 0;
	int iBaseNum = 0;
	int iCardNum = 0;
	bool bFind = false;
	while(m_bRunning)
	{
		FD_ZERO(&readset);
		FD_SET(m_iFd, &readset);
		tv.tv_sec = 0;
		tv.tv_usec = 1000 * 100;
		
		ret = select(m_iFd + 1, &readset, NULL, NULL, &tv);
		if(ret < 0)
		{
			LOGE("read port error");
			break;
		}
		
		if(ret == 0)
		{
			continue;
		}

		if(FD_ISSET(m_iFd, &readset))
		{
			size = read(m_iFd, szBuf + pos, 1);
			if(size > 0)
			{
				if(iState == 0)
				{
					if(szBuf[pos] == 0x30)
					{
						pos++;
						iState = 1;
					}
				}
				else if(iState == 1)
				{
					if(szBuf[pos] == 0x32)
					{
						pos++;
						iState = 2;
					}
					else
					{
						pos = 0;
						iState = 0;
					}
				}
				else if(iState == 2)
				{
					if(szBuf[pos] == 0x0D)
					{
						iState = 3;
					}
					pos++;
				}
				else if(iState == 3)
				{
					if(szBuf[pos] == 0x0A)
					{
						pos++;
						iState = 4;
					}
					else
					{
						pos = 0;
						iState = 0;
					}
				}
				else if(iState == 4)
				{
					//LOGE("CardNo%03d, pos=%d, %s", idx++, pos, szBuf + pos -18);
					/*char szTmp[2];
					int c = 0;
					for(int i = 0; i < 12; i += 2)
					{
						szTmp[0] = szBuf[pos -16 + i];
						szTmp[1] = szBuf[pos -16 + i + 1];
						sscanf(szTmp, "%02x", &c);
						printf("%d ", c);
					}
					*/
					szTmp[0] = szBuf[pos -10];
					szTmp[1] = szBuf[pos -9];
					sscanf(szTmp, "%02x", &iBaseNum);
					//iCardNum = iBaseNum * 100000;
					szTmp[0] = szBuf[pos -8];
					szTmp[1] = szBuf[pos -7];
					szTmp[2] = szBuf[pos -6];
					szTmp[3] = szBuf[pos -5];
					sscanf(szTmp, "%04x", &iCardNum);
					iCardNum = iBaseNum * 100000 + iCardNum;

					SendCardNumber(iCardNum, FarReader);
					
					//LOGE("get card num is %d", iCardNum);
					
					pos = 0;
					iState = 0;
				}
			}
		}
	}
	
	return 1;
}

// + is 43, . is 46
int BaseSerialPort::RelayVoltageRecverProcess()
{
	LOGE("enter RelayVoltageRecverProcess");
	char szBuf[32] = {0};
	fd_set readset; 
	struct timeval tv;
	int ret = -1;
	int size = 0;
	int pos = 0;
	int iState = 0;
	while(m_bRunning)
	{
		FD_ZERO(&readset);
		FD_SET(m_iFd, &readset);
		tv.tv_sec = 0;
		tv.tv_usec = 1000 * 1000;
		
		ret = select(m_iFd + 1, &readset, NULL, NULL, &tv);
		if(ret < 0)
		{
			LOGE("read port error");
			break;
		}
		
		if(ret == 0)
		{
			continue;
		}

		if(FD_ISSET(m_iFd, &readset))
		{
			size = read(m_iFd, szBuf + pos, 1);
			if(size > 0)
			{
				//LOGE("szBuf[%d] = %d,   szBuf is %s", pos, szBuf[pos], szBuf);
				if(iState == 0)
				{
					if(szBuf[pos] == '+')
					{
						pos = -1;
						iState = 1;
					}
				}
				else if(iState == 1)
				{
					if(szBuf[pos] == '+')
					{
						//此时的buf如12.03+，注意前边不带+，后边带+号
						szBuf[pos] = '\0';
						//LOGE("pos = %d, szBuf = %s, szBuf[3] = %d, szBuf[4] = %d", pos, szBuf, szBuf[3], szBuf[4]);
						double v = atof(szBuf);
						WNNotification notification;
						notification.mParams[0] = (int)v;//电压值整数部分
						notification.mParams[1] = szBuf[3] - 48;//电压值小数部分第一位
						notification.mParams[2] = szBuf[4] - 48;//电压值小数部分第二位
						notification.muNotificationId = WNNotificationCenter::DefaultCenter()->GetNotificationIdByName("GTS_RelayVoltage");
						WNNotificationCenter::DefaultCenter()->PostNotification(&notification);
						
						iState = 0;
						pos = 0;
					}
				}
				pos++;
			}
		}
	}
	
	return 1;
}

void BaseSerialPort::SendCardNumber(int number, SerialPortType type)
{
	WNNotification notification;
	if(type == NearReader)
	{
		notification.mParams[0] = 0;
	}
	else if(type == FarReader)
	{
		notification.mParams[0] = 1;
	}
	else
	{
		return;
	}
	notification.mParams[1] = number;
	notification.muNotificationId = WNNotificationCenter::DefaultCenter()->GetNotificationIdByName("GTS_CardNumber");	
	
	WNNotificationCenter::DefaultCenter()->PostNotification(&notification);
}