/*utf8解析行 Don't remove or modify this first line!*/
#ifndef WNDECOD_H
#define WNDECOD_H

class WNByteBuffer;

typedef struct _WNFrameInfo
{
    unsigned int uUserInfo[2]; //包类型
    unsigned int uHeader;       //头标记
    unsigned int uVersion;      //版本号
    unsigned int uSize;			//数据长度
    unsigned int uType;			//数据包类型
    unsigned int uStreamID;		//数据流ID，
    unsigned int uTimeStamp;	//时间戳
} WNFrameInfo, *PWNFrameInfo;


typedef struct _WNDecodedFrameInfo
{
    unsigned int uUserInfo[2];
    unsigned int uDataFormat; //数据格式
    unsigned int uWidth; //宽度
    unsigned int uHeight; //高度
    unsigned int uSize; //数据大小
    unsigned int uStreamID; //数据流ID
    unsigned int uTimeStamp; //时间戳
} WNDecodedFrameInfo, *PWNDecodedFrameInfo;

typedef const char* WNDecoderName;

class WNDecoder;

class WNDecoderDelegate
{
public:
    virtual ~WNDecoderDelegate() {  }
    virtual void OnNetworkVideoDeviceDecoded(WNDecoder* pDevice) {  }
};

class WNDecoder
{
public:
    //托管对象，非空时通过此对象回调WNDecoderDelegate的接口
    WNDecoderDelegate* mpDecoderDelegate;

public:
    WNDecoder() { mpDecoderDelegate = 0; }
    virtual ~WNDecoder() { mpDecoderDelegate = 0; }

    //返回解码器名称
    virtual WNDecoderName DecoderName() = 0;

    //初始化解码器
    //成功返回true，识别返回false
    virtual bool InitDecoder() = 0;

    //解码器开始工作
    //成功返回true，识别返回false
    virtual bool StartWork() =0;

    //解码器停止工作
    //成功返回true，识别返回false
    virtual bool EndWork() = 0;

    //解码pFrameBuffer
    //pInfo是pFrameBuffer的相关信息
    //成功返回true，失败返回false
    virtual bool DoDecode(WNByteBuffer* pFrameBuffer, WNFrameInfo* pInfo = 0) = 0;

    //获得最后一次解码成功的帧(解码后的数据)
    //返回的帧在调用FreeDecodedFrame之前当前解码器不允许使用
    //[输出] pInfo是解码后帧的信息指针。
    //[输入] bSafeCall表示是否是安全调用。
    //安全调用是指在托管对象的OnNetworkVideoDeviceDecoded被调用的时候在同一线程中调用本函数
    //安全调用时无需加锁
    //注意，对同一WNDecoder对象只能选择一种方式调用本函数，或者安全调用，或者非安装调用
    virtual void* GetNextDecodedFrame(WNDecodedFrameInfo* pInfo = 0, bool bSafeCall = true) = 0;


    //释放通过GetNextDecodedFrame获得的解码后的数据
    //释放后的数据可以被当前解码器使用
    virtual void FreeDecodedFrame(void* pDecodedFrame) = 0;
};

//通过默认的消息中心创建解码器
//发送：
//  参数1设置为要创建的解码器名称指针(WNDecoderName类型)
//  参数2设置为0
//返回(调用完毕后)：
//  参数1不变
//  参数2设置为创建的解码器指针，继承WNDecoder
#define NT_CreateDecoder "NT_CreateDecoder"

//通过默认的消息中心销毁解码器
//发送：
//  参数1设置为要销毁的解码器名称指针(WNDecoderName类型)
//  参数2设置为要销毁的解码器指针
//返回(调用完毕后)：
//  参数1不变
//  参数2设置为0，表示销毁成功，继承WNDecoder
#define NT_DestroyDecoder "NT_DestroyDecoder"


#endif