/*utf8解析行 Don't remove or modify this first line!*/
#ifndef WNNOTIFICATIONCENTER_H
#define WNNOTIFICATIONCENTER_H

#include "WNObject.h"
#include "WNCLoopList.h"
#include <string>
using namespace std;

class WNNotificationCenter;
class WNReadWriteLocker;
class WNCreticalSection;


//默认2个参数，可以重定义此宏，但是此宏不能为0
//重定义此宏时，在包含WNNotificationCenter.h之前定义
#ifndef DEF_NT_PARAMS_COUNT
#define DEF_NT_PARAMS_COUNT 3
#endif

//通知消息类
class WNNotification : public WNObject
{
public:
	WNNotification(unsigned int muParamsCount = DEF_NT_PARAMS_COUNT);
	virtual ~WNNotification();
	//通知名称
	virtual const char* NotificationName();
	//判断通知名称是否为指定名称
	virtual bool IsNotification(const char* lpNotificationName);
    //获得整个通知的大小
    virtual unsigned int GetSize();
    //获得整个通知的拷贝
    //返回值需要通过Release释放
    //未实现
    virtual WNNotification* GetCopy();
    //拷贝内容至pNotification
    //未实现
    virtual void CopyTo(WNNotification* pNotification);
public:
	unsigned int muNotificationId; //通知Id
    void* mpNotificationSender; //通知发送者
	WNObject* mpNotificationData; //通知数据
	WNNotificationCenter* mpNotificationCenter; //投递消息的消息中心
	unsigned int muParamsCount; //参数个数
	unsigned int mParams[DEF_NT_PARAMS_COUNT]; //参数列表
};

//通知接受者类
class WNNotificationReceiver
{
public:
	//通知回调
	virtual void OnReceiveNotification(WNNotification* pNotification) {  }
    virtual void OnReceiveNotification(WNNotification *pNotification, void* pThread) { }
	virtual ~WNNotificationReceiver(){}
};

class WNWorkThread;

//通知观察者类
class WNNotificationObserver : public WNNotificationReceiver
{
public:
	virtual ~WNNotificationObserver(){}
    //开始观察指定发送者以及消息中心的通知(字符串)
	//lpNotificationName是要观察的通知名称
	//pSender是发送者，为NULL时表示观察所有发送者发送的lpNotificationName
    //pNotification是指定的消息中心,为默认值NULL是表示使用WNNotificationCenter::DefaultCenter()
	//返回值为通知Id，返回值为非0表示成功，否则表示失败
	virtual unsigned int StartObserveNotification(const char* lpNotificationName, void* pSender = NULL, WNNotificationCenter* pNotificationCenter = NULL);
	
    //停止观察指定发送者以及消息中心的通知(字符串)
	//lpNotificationName是要停止观察的消息名称
	//pSender是发送者，为NULL时表示停止观察所有发送者发送的lpNotificationName所对应的消息
	//当lpNotificationName为NULL时表示停止观察所有的消息
    //pNotification是指定的消息中心,为默认值NULL是表示使用WNNotificationCenter::DefaultCenter()
    virtual void StopObserveNotification(const char* lpNotificationName = NULL, void* pSender = NULL, WNNotificationCenter* pNotificationCenter = NULL);


    //开始在指定线程观察指定发送者以及消息中心的通知(字符串)
    //lpNotificationName是要停止观察的消息名称
    //pThread是指定的线程，为NULL时表示在当前线程观察
    //pSender是发送者，为NULL时表示停止观察所有发送者发送的lpNotificationName所对应的消息
    //当lpNotificationName为NULL时表示停止观察所有的消息
    //pNotification是指定的消息中心,为默认值NULL是表示使用WNNotificationCenter::DefaultCenter()
    //返回值为通知Id，返回值为非0表示成功，否则表示失败
    virtual unsigned int StartObserveNotificationOnThread(const char *lpNotificationName, WNWorkThread* pThread = NULL, void *pSender = NULL, WNNotificationCenter *pNotificationCenter = NULL);

    //停止在指定的线程观察指定发送者以及消息中心的通知(字符串)
    //lpNotificationName是要停止观察的消息名称
    //pThread是指定的线程，为NULL时表示在当前线程观察
    //pSender是发送者，为NULL时表示停止观察所有发送者发送的lpNotificationName所对应的消息
    //当lpNotificationName为NULL时表示停止观察所有的消息
    //pNotification是指定的消息中心,为默认值NULL是表示使用WNNotificationCenter::DefaultCenter()
    virtual void StopObserveNotificationOnThread(const char* lpNotificationName = NULL, WNWorkThread* pThread = NULL, void* pSender = NULL, WNNotificationCenter* pNotificationCenter = NULL);

};

//WNObserverInfo用于保存观察者的信息
class WNObserverInfo
{
public:
	WNObserverInfo();
	virtual ~WNObserverInfo(){}
public:
	WNNotificationReceiver* mpNotificationReceiver; //通知接受者
	void* mpNotificationSender; //通知发送者
	bool mbIsValid; //是否有效
};

//WNNotificationObserverList用于保存指定通知的观察者信息列表
class WNNotificationObserverList : public WNCLoopList<WNObserverInfo>
{
public:
	WNNotificationObserverList();
	virtual ~WNNotificationObserverList();
	//添加观察者
	void AddObserver(WNNotificationObserver* pObserver, void* pSender);
	//移除观察者
	void RemoveObserver(WNNotificationObserver* pObserver, void* pSender);
	//投递通知
	void PostNotification(WNNotification* pNotification, void* pSender = NULL);
public:
	unsigned int muNotificationId; //通知Id
	string mstrNotificationName; //通知名称
	WNCreticalSection* mpLocker;
protected:
	unsigned int muNotificationLevel; //同一线程中PostNotification函数调用的层次
	unsigned int muRemoveCount; //要删除的观察者数目，用于同一线程的通知中调用RemoveObserver函数
};

//WNNotificationCenter是通知中心类
//默认会存在一个实例，通过WNNotificationCenter::DefaultCenter()获得
class WNNotificationCenter : public WNObject
{
public:
	WNNotificationCenter();
	virtual ~WNNotificationCenter(void);
	//获得默认通知中心实例
	static WNNotificationCenter* DefaultCenter();
	//注册通知
	//返回通知id
	//如果通知未注册则返回新的id，如果已注册则返回已有id
	virtual unsigned int RegisterNotification(const char* lpNotificationName);
	//根据通知名称获得通知id
	//如果指定的名称没有注册，则返回0，已注册返回非0的id
	virtual unsigned int GetNotificationIdByName(const char* lpNotificationName);
	//根据通知id得到通知名称
	virtual const char* GetNotificationNameById(unsigned int uNotificationId);
	//添加观察者
	//pObserver是要添加的观察者
	//lpNotificationName是要观察的消息名称
	//pSender是指定的消息发送者，如果为NULL则观察来自所有观察者的lpNotificationName
	//返回值为要观察的消息Id
	virtual unsigned int AddObserver(WNNotificationObserver* pObserver, const char* lpNotificationName, void* pSender = NULL);
	//移除观察者
	//pObserver是要移除的观察者
	//lpNotificationName是指定的通知，为NULL表示移除此观察者所有的通知，否则移除与lpNotificationName以及pSender匹配的观察者
	//pSender是消息的发送者，当lpNotificationName为NULL时被忽略
	//       为NULL时表示停止观察所有发送者发送的lpNotificationName所对应的消息
	virtual void RemoveObserver(WNNotificationObserver* pObserver, const char* lpNotificationName = NULL, void* pSender = NULL);

	//投递通知，低效方法，投递之前会先将lpNotificationName转换为Id
	//lpNotificationName是通知名称
	//pNotificationData是要投递的数据
	//pSender是发送者
	//成功返回TRUE，失败返回FALSE
	virtual bool PostNotification(const char* lpNotificationName, WNObject* pNotificationData = NULL, void* pSender = NULL);
	//投递通知，高效方法
	//lpNotificationName是通知Id
	//pNotificationData是要投递的数据
	//pSender是发送者
	//成功返回TRUE，失败返回FALSE
	virtual bool PostNotification(unsigned int uNotificationId, WNObject* pNotificationData = NULL, void* pSender = NULL);

	//投递通知，最高效方法
	//pNotification是要投递的通知，投递之前设置muNotificationId以及mpNotificationData即可
	//pSender是发送者
	//成功返回TRUE，失败返回FALSE
	//由于此方法中的pNotification参数可以使用栈上分配的对象，因此效率最高
	//需要注意的是，接收者对pNotification进行Retain操作可能会由于是在栈上分配而导致不可预料的结果
	//此时可以检测pNotification的mbIsAllocOnHeap成员来判断pNotification是否为栈上分配的
	virtual bool PostNotification(WNNotification* pNotification, void* pSender = NULL);

protected:
	WNNotificationObserverList** mppNotificationObserverList; //消息观察者列表数组
	unsigned int muNotificationListMaxCount; //消息观察者列表数组最大数量
	unsigned int muNextNotificationId; //下一次注册时使用的消息id
	WNReadWriteLocker* mpReadWriteLocker; //读写锁
};

#endif
