/*utf8解析行 Don't remove or modify this first line!*/
#ifndef WNOBJECT_H
#define WNOBJECT_H
#ifndef NULL
#ifdef __cplusplus
#define NULL    0
#else
#define NULL    ((void *)0)
#endif
#endif

class WNObject;
class WNCreticalSection;
class WNObjectDelegate;

template <class T> class  WNSmartPtr;     

class WNObjectTypeFnBase
{
public:
    virtual ~WNObjectTypeFnBase() {  }
    virtual void DoDestroy(void* pDes) const { }
};

template <typename T>
class WNObjectTypeFn : public WNObjectTypeFnBase
{
public:
    virtual ~WNObjectTypeFn() {  }
    T* DoConvert(void* pDes) const
    {
        return (T*)(pDes);
    }
    virtual void DoDestroy(void* pDes) const
    {
        delete DoConvert(pDes);
    }
};

template <typename T>
static WNObjectTypeFn<T>* getWNObjectTypeFnInstance()
{
    static WNObjectTypeFn<T> mInstance;
    return &mInstance;
}

//只能以指针形式使用
class WNObject
{
public:
	WNObject();
	virtual ~WNObject();
	//初始化计数器
	virtual void ResetRefInfo();
	//初始化计数器以及设置托管对象
	virtual void ResetRefInfoAndDelegate(WNObjectDelegate* pDelegate = NULL);
	//设置对象的安全锁
	//为空时表示创建安全锁
	//否则使用传入的安全锁
	virtual void SetSafeLocker(WNCreticalSection* pLocker = NULL);
	//获得对象的安全锁
	WNCreticalSection* GetSafeLocker();
	//取消安全锁
	virtual void ResetSafeLocker();
	//增加对象的引用
	//返回对象的this指针
	virtual WNObject* Retain();
	//减少对象的引用
	//bClearDelayRelease表示是否清空延迟减少计数器
	//返回对象的this指针，如果对象已销毁则返回NULL
	virtual WNObject* Release(bool bClearDelayRelease = true);
	//自动释放
	virtual WNSmartPtr<WNObject*> AutoRelease();
	//延迟减少对象的引用(增加延迟减少计数器)
	//返回对象的this指针
	virtual WNObject* DelayRelease();
	//处理延迟减少计数器
	//如果bOnlySetZero为TRUE，则仅仅将延迟引用计数器置零，
	//默认会做如下操作:
	//miRetainCount -= miDelayReleaseCount
	//如果当前对象引用为0，则会调用delete操作
	virtual WNObject* ClearDelayRelease(bool bOlnySetZero = false);
	//当前的引用计数器的值
	inline int RetainCount() { return miRetainCount; }
	//当前延迟引用计数器的值
	inline int DelayCount() { return miDelayReleaseCount; }
	//当前实际的引用计数器
	inline int RefCount() { return miRetainCount - miDelayReleaseCount; }
	//设置或者或其托管对象
	inline WNObjectDelegate*& Delegate() { return mpDelegate; }
public:
	int miRetainCount : 31; //引用计数器
	bool mbIsAllocOnHeap : 1; //是否在堆上分配
	int miDelayReleaseCount; //延迟减少计数器
	WNObjectDelegate* mpDelegate; //托管对象
	WNCreticalSection* mpLoker; //安全锁
	bool mbIsOwnLocker; //安全锁是否属于自己
    void* mpThis; //当前对象new出来的指针
    WNObjectTypeFnBase* mpTypeFn; //用于析构mpThis的对象
};


typedef WNObject* obj;

//WNObject*类型的智能指针，T只允许为指针类型
template <class T>
class WNSmartPtr
{
public:
	T ptr;
public:
	inline WNSmartPtr() { ptr = 0; }
	inline WNSmartPtr(const WNSmartPtr<T>& sPtr):ptr(0) { *this = sPtr; }
	inline WNSmartPtr(const T& pPtr):ptr(0) { *this = (obj&)pPtr; }
	virtual ~WNSmartPtr(){}
	template <typename Ptr>
	inline WNSmartPtr(const Ptr pPtr):ptr(0) { *this = (obj&)pPtr; }
	virtual bool isEqualTo(const obj& pPtr) const
	{
		return pPtr == (obj)ptr;
	}
	template <typename Ptr>
	inline bool operator == (const Ptr& pPtr) const
	{
		return isEqualTo((obj&)pPtr);
	}
	inline bool operator == (const WNSmartPtr<T>& sPtr) const
	{
		return *this == (obj&)(sPtr.ptr);
	}
	template <typename Ptr>
	inline bool operator != (const Ptr& pPtr) const
	{
		return (*this == (obj)pPtr)?false:true;
	}
	inline bool operator != (const WNSmartPtr<T>& sPtr) const
	{
		return *this != sPtr.ptr;
	}
	WNSmartPtr<T>& operator = (const obj& pPtr)
	{
		pPtr->Retain();
		if (ptr)
		{
			pPtr->Release();
			ptr = NULL;
		}
		ptr = (T&)pPtr;
		return *this;
	}
	template <typename Ptr>
	inline WNSmartPtr<T>& operator = (const Ptr& pPtr) { return *this = (obj&)pPtr; }
	inline WNSmartPtr<T>& operator = (const WNSmartPtr<T>& sPtr) { return *this = (obj&)(sPtr.ptr); }
	template <typename ToType>
	inline operator ToType& () const { return (ToType&)ptr; }
	template <typename ToType>
	inline operator ToType () { return (ToType)ptr; }
	inline operator obj& () const { return (obj&)(ptr); }
	inline operator obj () { return (obj)(ptr); }
	inline T& ref() { return (T&)ptr; }
};


class WNObjectDelegate
{
public:
	//当WNObject的muRetainCount以及muDelayReleaseCount的差变为0时触发
	//如果返回值为TRUE则会delete此对象
	virtual bool OnWNObjectRetainCountIsZero(WNObject* pObject) { return true; }
	virtual ~WNObjectDelegate(){}
};

 
//只能以指针形式使用
template <class T>
class WNObjectEx : public WNObject
{
public:
	T dataRef;
};


template <typename WNObjectClass>
WNObjectClass* _WNAlloc()
{
	WNObjectClass* pObject = new WNObjectClass;
	pObject->mbIsAllocOnHeap = true;
    pObject->mpThis = pObject;
    pObject->mpTypeFn = getWNObjectTypeFnInstance<WNObjectClass>();
	return pObject;
}

template <typename WNObjectClass>
WNObjectClass* _WNSafeAlloc()
{
	WNObjectClass* pObject = new WNObjectClass;
	pObject->SetSafeLocker();
	pObject->mbIsAllocOnHeap = true;
    pObject->mpThis = pObject;
    pObject->mpTypeFn = getWNObjectTypeFnInstance<WNObjectClass>();
	return pObject;
}

#ifndef WNAlloc
#define WNAlloc(WNObjectClass) _WNAlloc<WNObjectClass>()
#endif

#ifndef WNSafeAlloc
#define WNSafeAlloc(WNObjectClass) _WNSafeAlloc<WNObjectClass>()
#endif


//#define doRelease(ppObj) _doRelease((WNObject**)(&(ppObj)))
//void _doRelease(WNObject** ppObj);

//#define doAutoRelease(ppObj) _doDelayRelease((WNObject**)(&(ppObj)))
//void _doDelayRelease(WNObject** ppObj);

#define doRelease(pObj) \
    if (pObj) \
    { \
        pObj->Release(); \
        pObj = 0; \
    }

#define doDelayRelease(pObj) \
    if (pObj) \
    { \
        pObj->DelayRelease(); \
        pObj = 0; \
    }

//template <typename T>
//void doRelease(T*& pObjRef)
//{
//    pObjRef->Release();
//    pObjRef = NULL;
//}
//
//template <typename T>
//void doDelayRelease(T*& pObjRef)
//{
//    pObjRef->DelayRelease();
//    pObjRef = NULL;
//}

#endif
