/*utf8解析行 Don't remove or modify this first line!*/
#ifndef WNSOCKETOBSERVER_H
#define WNSOCKETOBSERVER_H

#include "WNCLoopList.h"
#include "WNCreticalSection.h"

#include <string>
using namespace std;
class WNSocketObserverTarget;

#ifndef INVALID_SOCKET
#define INVALID_SOCKET -1
#endif

#ifndef SD_BOTH
#define SD_BOTH 2
#endif

#ifndef SOCKET_ERROR
#define SOCKET_ERROR (-1)
#endif

class IWNSocketObserver
{
public:
	virtual bool AddSocket(WNSocketObserverTarget* socketInfo) = 0;
	virtual bool RemoveSocket(WNSocketObserverTarget* socketInfo) = 0;
	virtual bool StartServer() = 0;
	virtual bool StopServer() = 0;
	virtual const char* GetServerName() = 0;
	virtual ~IWNSocketObserver(){}
	virtual bool IsWorking() = 0;
};

typedef WNCLoopList<IWNSocketObserver*> WNSocketObserverList;

class WNSocketObserverTargetDelegate
{
public:
	//当前socket可读(或可以进行accept操作)
	virtual void OnObserverRead(IWNSocketObserver* observer) = 0;
	//当前socket错误
	virtual void OnObserverError(IWNSocketObserver* observer) = 0;
	virtual ~WNSocketObserverTargetDelegate(){}
};

class WNSocketObserverTarget
{
public:
	WNSocketObserverTarget();
	virtual ~WNSocketObserverTarget();
	enum ObserverState
	{
		None, //不在观察者server中
		Adding, //正在添加至观察者列表
		Observering, //正在观察
		Removing //正在从观察者列表中移除
	};

	enum SocketState
	{
		SS_None, //无状态，无有效socket
		SS_Normal, //正常状态，可以读写
		SS_Listening, //侦听状态
		SS_Accepting, //等待accept
		SS_Connecting, //连接状态
		SS_Reading, //读状态
		SS_Writing, //写状态
		SS_Closing //正在关闭
	};
	virtual void CreateLocker();
	virtual void DestroyLocker();
	virtual bool TryLock();
	virtual bool Lock();
	virtual bool Unlock();
public:
	int mSocket;
	volatile SocketState mSocketState;
	string mObserverName;
	//CRITICAL_SECTION* mpLocker;
	WNCreticalSection* mpLocker;
    unsigned long mlLockRef;
	unsigned long mdwLockedThreadId;
	WNSocketObserverTargetDelegate* mpObserverTargetDelegate;
	int miObserverErrorCode;
	pthread_t mLockedThreadId;
};



typedef WNCLoopList<WNSocketObserverTarget*> WNSocketObserverTargetList;





#endif

