#include "WNMemoryPool.h"
#include <string.h>

WNMemoryPool::WNMemoryPool(int size)
:m_iBufSize(size)
,m_iWriteId(0)
,m_iReadId(0)
{
	m_pBuf = new unsigned char[size];
	memset(m_pBuf, 0, size);
    pthread_mutex_init(&m_mtx, NULL);
}

WNMemoryPool::~WNMemoryPool()
{
    pthread_mutex_destroy(&m_mtx);
	delete []m_pBuf;
	m_pBuf = NULL;
}

void WNMemoryPool::Reset()
{
    pthread_mutex_lock(&m_mtx);
	m_iWriteId = 0;
	m_iReadId = 0;
    pthread_mutex_unlock(&m_mtx);
}

int WNMemoryPool::Write(unsigned char * data, int writelen)
{
    pthread_mutex_lock(&m_mtx);
    int buf_unused_len = 0;
    if (m_iWriteId >= m_iReadId)
    {
        buf_unused_len = m_iBufSize - m_iWriteId + m_iReadId;
    }
    else
    {
        buf_unused_len = m_iReadId - m_iWriteId;
    }

    if (buf_unused_len >= writelen)
    {
        if (m_iWriteId >= m_iReadId)
        {
            int temp = m_iBufSize - m_iWriteId;//计算右边的可用长度
            if (temp >= writelen)
            {
                memcpy(m_pBuf + m_iWriteId, data, writelen);
                m_iWriteId += writelen;
            }
            else
            {
                memcpy(m_pBuf + m_iWriteId, data, temp);
                memcpy(m_pBuf, data + temp, writelen - temp);
                m_iWriteId = writelen - temp;
            }
        }
        else
        {
            memcpy(m_pBuf + m_iWriteId, data, writelen);
            m_iWriteId += writelen;
        }
        pthread_mutex_unlock(&m_mtx);

        return writelen;
    }
    else
    {
        LOGE("memory pool for audio is not enought, datalen: %d, buf_unused_len: %d", writelen, buf_unused_len);
    }
    // 空间不够，直接丢掉
    pthread_mutex_unlock(&m_mtx);

    return 0;
}

int WNMemoryPool::Read(unsigned char * pOutData, int iRequestLen)
{
    pthread_mutex_lock(&m_mtx);
    int valid_data_len = 0;
    if (m_iWriteId >= m_iReadId)
    {
        valid_data_len = m_iWriteId - m_iReadId;
    }
    else
    {
        valid_data_len = m_iBufSize - m_iReadId + m_iWriteId;
    }

    if (valid_data_len >= iRequestLen)
    {
    	if (m_iWriteId >= m_iReadId)
    	{
    		memcpy(pOutData, m_pBuf + m_iReadId, iRequestLen);
    		m_iReadId += iRequestLen;
    	}
    	else
    	{
    		int temp = m_iBufSize - m_iReadId ;
    		if (temp >= iRequestLen)
    		{
    			memcpy(pOutData, m_pBuf + m_iReadId, iRequestLen);
    			m_iReadId += iRequestLen;
    		}
    		else
    		{
    			memcpy(pOutData, m_pBuf + m_iReadId, temp);
    			memcpy(pOutData + temp, m_pBuf, iRequestLen - temp);
    			m_iReadId = iRequestLen - temp;
    		}
    	}
	    pthread_mutex_unlock(&m_mtx);

		return iRequestLen;
    }

    pthread_mutex_unlock(&m_mtx);

	return 0;
}